// Class: ColorExample
//
// Author: Alyce Brady
//
// License Information:
//   This class is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation.
//
//   This class is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

import java.util.Collection;
import java.util.Iterator;

import edu.kzoo.grid.BoundedGrid;
import edu.kzoo.grid.ColorBlock;
import edu.kzoo.grid.Grid;
import edu.kzoo.grid.Location;
import edu.kzoo.grid.display.ColorBlockDisplay;
import edu.kzoo.grid.display.DisplayMap;
import edu.kzoo.grid.gui.ColorChoiceDDMenu;
import edu.kzoo.grid.gui.ControlButton;
import edu.kzoo.grid.gui.EnabledDisabledStates;
import edu.kzoo.grid.gui.GridAppFrame;
import edu.kzoo.grid.gui.nuggets.ColorChoiceMenu;
import edu.kzoo.grid.gui.nuggets.MinimalFileMenu;
import edu.kzoo.util.NamedColor;

/**
 *  Grid Package Examples:<br>
 *
 *    The ColorExample class constructs a simple graphical user interface
 *    that displays various colors defined in the NamedColor class. 
 *
 *  @author Alyce Brady
 *  @version Sep 4, 2004
 **/
public class ColorExample extends GridAppFrame
{
    private static final boolean DISPLAY_GRID_AFTER_BUTTON_PRESSES = true;

    private ColorChoiceMenu standardMenu;
    private ColorChoiceMenu alternateMenu;
    private ColorChoiceMenu currentMenu;

    public static void main(String[] args)
    {
        ColorExample example = new ColorExample();
    }

    public ColorExample()
    {
        includeMenu(new MinimalFileMenu());

        // Create a button that will show all the constants in NamedColor.
        includeControlComponent(
            new ControlButton(this, "Show NamedColor Constants", DISPLAY_GRID_AFTER_BUTTON_PRESSES)
                {   public void act() {showNamedColorConstants();}},
            EnabledDisabledStates.NEEDS_GRID_AND_APP_WAITING);

        // Create a standard color choice menu, specifying the default color.
        // Add another color to it.
        standardMenu =
            new ColorChoiceMenu("Standard Color Menu (Plus Burgundy)", "Violet")
                {   public void act() { currentMenu = standardMenu; }  };
        standardMenu.addColorChoice(new NamedColor(192, 10, 72, "Burgundy"));
        includeControlComponent(standardMenu,
                                    EnabledDisabledStates.NEEDS_APP_WAITING);
        currentMenu = standardMenu;

        // Create a customized color choice menu, specifying the default color.
        ColorChoiceDDMenu.ColorChoice defaultColor =
            new ColorChoiceDDMenu.ColorChoice("Cyan", NamedColor.CYAN);
        ColorChoiceDDMenu.ColorChoice[] customMenu =
            {
              defaultColor,
              new ColorChoiceDDMenu.ColorChoice("Fuschia", NamedColor.FUSCHIA),
              new ColorChoiceDDMenu.ColorChoice("Pink", NamedColor.ROSE),
              new ColorChoiceDDMenu.ColorChoice("Bright Green",
                                                NamedColor.GREEN),
              new ColorChoiceDDMenu.ColorChoice("Dark Green",
                                                NamedColor.DARK_GREEN),
              new ColorChoiceDDMenu.ColorChoice("Dark Blue",
                                                NamedColor.DARK_BLUE),
              new ColorChoiceDDMenu.ColorChoice("Midnight",
                                                NamedColor.MIDNIGHT),
              new ColorChoiceDDMenu.ColorChoice("Lilac", NamedColor.LILAC),
              new ColorChoiceDDMenu.ColorChoice("Brown", NamedColor.BROWN),
            };
        alternateMenu =
            new ColorChoiceMenu("Alternate Color Menu", customMenu,
                                defaultColor)
                {   public void act() { currentMenu = alternateMenu; }  };
        includeControlComponent(alternateMenu,
                                    EnabledDisabledStates.NEEDS_APP_WAITING);

        // Construct the window.  Specify how to display color blocks.
        // Have the display show object toString information in tool tips, not
        // just grid locations.
        constructWindowContents("Color Example with Clickable Cells",
                                    NamedColor.WHITE, 
                                    400, 400, 10);
        DisplayMap.associate("edu.kzoo.grid.ColorBlock",
                             new ColorBlockDisplay());
        getDisplay().makeToolTipsReportObject();

        // Create the grid.
        Grid grid = new BoundedGrid(10, 10);
        setGrid(grid);
    }

    public void showNamedColorConstants()
    {
        Grid grid = getGrid();
        grid.removeAll();
        Collection namedColors = NamedColor.getAllNamedColors();
        Iterator it = namedColors.iterator();
        for ( int row = 0; row < grid.numRows(); row++ )
            for ( int col = 0; col < grid.numCols(); col++ )
            {
                Location loc = new Location(row, col);
                if ( it.hasNext() )
                {
                    NamedColor c = (NamedColor) it.next();
                    System.out.print(c + ": ");
                    System.out.println(c.getRGBRepresentation());
                    new ColorBlock(c, grid, loc);
                }
            }
        showGrid();
    }

    /** Handles a mouse press over the grid display.  Creates a color block
     *  in the specified location if there isn't one there already, or
     *  removes the current color block if there is one.  The color of
     *  new color blocks is determined by the current choice in the color
     *  choice menu.
     **/
    protected void onMousePressOverDisplay(Location loc)
    {
        if ( getGrid().isEmpty(loc) )
            getGrid().add(new ColorBlock(currentMenu.currentColor()), loc);
        else
            getGrid().remove(loc);
        showGrid();
    }

}
