// Class: MBSEnvEditor
//
// Author: Alyce Brady
//
// This class is based on the College Board's EnvironmentController 
// and FishToolbar classes, as allowed by the GNU General Public License.
// EnvironmentController and FishToolbar are black-box GUI classes
// within the AP(r) CS Marine Biology Simulation case study 
// (see www.collegeboard.com/ap/students/compsci).
//
// License Information:
//   This class is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation.
//
//   This class is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

package edu.kzoo.mbsInAGrid.gui;

import java.awt.Color;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JToolBar;

import edu.kzoo.grid.Direction;
import edu.kzoo.grid.Grid;
import edu.kzoo.grid.GridObject;
import edu.kzoo.grid.Location;
import edu.kzoo.grid.gui.ColorChoiceDDMenu;
import edu.kzoo.grid.gui.GridAppFrame;
import edu.kzoo.grid.gui.GridEditor;
import edu.kzoo.grid.gui.GridPkgFactory;

import edu.kzoo.mbsInAGrid.GridFish;

/**
 *  Grid Container Package Examples:<br>
 *
 *  The <code>MBSEnvEditor</code> class provides a window in which
 *    to edit an environment for the Marine Biology Simulation program.
 *
 *  @author Alyce Brady
 *  @version 1 December 2002
 **/
public class MBSEnvEditor extends GridEditor
{
    // this is the direction each new fish will face when created
    private static final Direction START_FACING = Direction.EAST;

    // encapsulated state for this MBSEnvEditor object
    protected ColorChoiceDDMenu colorMenu;

    /** Creates an MBS Environment Editor.
     *  Use the constructWindowContents method to set the other properties of
     *  the window and make it visible.
     *    @param env   the environment to edit
     *    @param frame  the frame that invoked this environment editor
     **/
    public MBSEnvEditor(GridAppFrame frame)
    {
        super(frame);
    }


  // methods for building editing palette

    /** Puts tools for choosing environment object attributes in
     *  the editing palette, in particular a Type tool for choosing
     *  the type of environment object and a Color tool for choosing
     *  the object's color.
     **/
    protected void fillPalette(JToolBar palette)
    {
        super.fillPalette(palette);

        palette.addSeparator();
        
        palette.add(new JLabel(" Color: "));
        colorMenu = new ColorChoiceDDMenu();
        enableDisableColorMenu();
        palette.add(colorMenu);
    }

    /** Builds up the list of environment object type choices for the editing
     *  palette.
     **/
    protected void addChoicesFromFactory(JComboBox cb)
    {
        super.addChoicesFromFactory(cb);
        int itemCount = cb.getItemCount();
        for ( int i = 0; i < itemCount; i++ )
        {
            GridObjectChoice item = (GridObjectChoice) cb.getItemAt(i);
            Class gridFishClass = edu.kzoo.mbsInAGrid.GridFish.class;
            if ( item.getObjectClass().equals(gridFishClass) )
                cb.setSelectedItem(item);
        }
    }


  // methods for handling user events
    
    /** Follows up when the user picks a new choice from the
     *  grid object combo box (specified listener action).
     **/
    protected void chooseGridObjType()
    {
        enableDisableColorMenu();
    }
    
    /** Enables or disables the color menu based on whether the currently
     *  selected environment object class has a constructor that takes a
     *  color parameter.
     **/
    protected void enableDisableColorMenu()
    {
        Class chosenType = currentGridObjectClass();
        boolean classHasColorParam = GridPkgFactory.hasFourArgCtor(chosenType);
        colorMenu.setEnabled(classHasColorParam);
    }

    /** Handles a mouse press over the grid display, editing the
     *  contents of the grid at the specified location.  If the
     *  location is empty, a new object of the currently specified
     *  grid object class will be added.  If the location is not
     *  empty, the object will be rotated or, if it has already
     *  rotated through all of the appropriate directions, the
     *  object will be removed.  The appropriate directions for
     *  a grid object are those that face adjacent neighboring
     *  locations, as defined by the grid.
     *    @see #currentGridObjectClass()
     *    @see edu.kzoo.grid.Grid#numAdjacentNeighbors()
     **/
    protected void onMousePressOverDisplay(Location loc)
    {
        if ( loc != null )
        {
            // If there's an object there of the wrong type, remove it.
            GridFish f = (GridFish)verifyType(getGrid().objectAt(loc));

            // If there's no object there, add one.  Otherwise rotate
            // it or, if it has come full circle, remove it.
            if ( f == null )
            {
                makeObject(loc, START_FACING);
            }
            else
            {
                // Rotate or remove object that is there; start by removing.
                getGrid().remove(f);

                // If we haven't come full circle, add it back rotated.
                Direction newFacing = getRotatedDirectionFor(f);
                if ( ! newFacing.equals(START_FACING) )
                    makeObject(loc, newFacing);
            }
            getDisplay().repaint();
        }
    }

    /** Verifies that the specified object is of the currently chosen
     *  type and removes it from the grid if it is not.
     *    @param obj   the grid object to verify
     **/
    protected GridObject verifyType(GridObject obj)
    {
        if ( obj != null && ! obj.getClass().equals(currentGridObjectClass()) )
        {
            getGrid().remove(obj);
            return null;
        }
        return obj;
    }

    /** Returns the object's new direction after rotating. **/
    protected Direction getRotatedDirectionFor(GridFish f)
    {
        int turningAngle =
                    Direction.FULL_CIRCLE/getGrid().numAdjacentNeighbors();
        return f.direction().toRight(turningAngle);
    }

    /** Constructs the specified type of object.
     *    @param loc   the location at which to create the object
     *    @param dir   the object's direction
     **/
    protected void makeObject(Location loc, Direction dir)
    {
        makeObject(currentGridObjectClass(), getGrid(), loc, dir,
                   colorMenu.currentColor());
    }
    
    /** Constructs the specified type of object.
     *    @param cls   the type of object to create
     *    @param grid  the grid in which to create the object
     *    @param loc   the location at which to create the object
     *    @param dir   the object's direction
     *    @param color the color of the new object (some objects may
     *                 ignore this)
     **/
    protected void makeObject(Class cls, Grid grid, Location loc,
                              Direction dir, Color color)
    {
        if (GridPkgFactory.hasFourArgCtor(cls))
            GridPkgFactory.constructGridObject(cls, grid, loc, dir, color);
        else
            GridPkgFactory.constructGridObject(cls, grid, loc, dir);
    }

}