// Class MBSMenuActionHandler
//
// Author: Alyce Brady
//
// This class is based on the College Board's EnvironmentController class,
// as allowed by the GNU General Public License.  EnvironmentController
// is a black-box GUI class within the AP(r) CS Marine Biology Simulation
// case study (see www.collegeboard.com/ap/students/compsci).
//
// License Information:
//   This class is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation.
//
//   This class is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

package edu.kzoo.mbsInAGrid.gui;

import java.awt.Toolkit;
import javax.swing.JOptionPane;
import javax.swing.JFileChooser;

import edu.kzoo.grid.Grid;
import edu.kzoo.grid.gui.SteppedGridAppFrame;
import edu.kzoo.grid.gui.FileMenuActionHandler;
import edu.kzoo.grid.gui.GridFileChooser;

import edu.kzoo.util.RandNumGenerator;

/**
 *  Grid Container Package Examples:<br>
 *
 *    The <code>MBSMenuActionHandler</code> class implements the methods
 *    used by the File and Seed menus defined in <code>MBSWindow</code>.
 *
 *  @author Alyce Brady (based on code by Julie Zelenski)
 *  @version 15 February 2004
 **/
public class MBSMenuActionHandler extends FileMenuActionHandler
{
    // constants for the three different seeding options
    public static final int DONT_RESET_SEED = 0, FIXED_SEED = 1, PROMPT_SEED = 2;

    private int seed, seedOption;
        
    /** Creates a new MBS menu action handler tied to the specified frame.
     *  (Precondition: frame is not null. )
     *  @param frame  the frame that uses this controller
     **/
    public MBSMenuActionHandler(SteppedGridAppFrame frame)
    {
        super(frame, new MBSDataFileHandler());
    }

    /** Indicates whether this controller supports grid editing.
     *  @return <code>true</code>
     **/
    public boolean supportsGridEditing()
    {
        return true; 
    }

    /** Invokes the appropriate environment editor.
     *    @param env   the environment (represented as a grid) to edit
     **/
    protected void invokeEditor()
    {
        resetSeedIfNeeded();
        MBSEnvEditor editor = new MBSEnvEditor(getParentFrame());
        editor.constructWindowContents();
    }
    
    /** Enables editing of an existing environment grid.
     **/
    public void editGrid(Grid grid)
    {
        ((SteppedGridAppFrame)getParentFrame()).stop();
        super.editGrid();
    }

    /** Reads a new environment from a saved data file. Gets file via
     *  file chooser dialog, reads contents using MBSDataFileHandler.
     *  On error, brings up dialog and continues using previous environment.
     **/
    public void openFile()
    {
        GridFileChooser chooser = getFileChooser();
        if (chooser.showOpenDialog(getParentFrame()) !=
                                                JFileChooser.APPROVE_OPTION)
            return;
            
        Grid newEnv;
        try 
        {
            resetSeedIfNeeded();
            MBSDataFileHandler handler = (MBSDataFileHandler) getFileHandler();
            handler.setBoundedRepresentation(chooser.boundedClass());
            handler.setUnboundedRepresentation(chooser.unboundedClass());
            newEnv = handler.readGrid(chooser.getSelectedFile());
        }
        catch (Exception ex)
        {
            JOptionPane.showMessageDialog(getParentFrame(),
                                    "Unable to read environment from file.\n"
                                    + "Reason: " + ex, "Error reading file",
                                    JOptionPane.ERROR_MESSAGE);
            return;
        }
        getParentFrame().setGrid(newEnv);
    }
            
    /** Changes the current run option to one of DONT_RESET, FIXED_SEED, or
     *  PROMPT_SEED to specify the seed behavior.   The seed option now
     *  in effect is returned. Thist usually will be the same as newOption
     *  but in cases where the change failed to go through (user cancelled
     *  dialog), the retun value will be the existing unchanged option.
     *  @param newOption the seed option to use for new/open environments
     *  @returns the seed option settled on
     **/
    public int setSeedOption(int newOption)
    {
        if (newOption == FIXED_SEED && !promptUserForSeed())
            return seedOption;	// leave option unchanged
        seedOption = newOption;
        if (seedOption == PROMPT_SEED)
            JOptionPane.showMessageDialog(getParentFrame(), 
            "You will be prompted for a new seed when opening or creating an environment.", 
            "Prompt for seed", JOptionPane.INFORMATION_MESSAGE);
        return seedOption;
    }
    
    /** If seed is being reset for each new/open environment, this
     *  method handles re-setting the generator appropriately.
     **/
    private void resetSeedIfNeeded()
    {
        switch (seedOption) 
        {
            case PROMPT_SEED:
                promptUserForSeed(); // fall-through to next case
            case FIXED_SEED:
                RandNumGenerator.getInstance().setSeed(seed); 
        }
    }


    /** Runs a dialog asking the user to input a new seed. If
     * successful, updates seed and returns true, else returns false.
     **/
    private boolean promptUserForSeed() 
    {
        // dialog suggests 17 because that's the value mentioned in the narrative
        int suggestion = seed == 0 ? 17 : seed;
        String seedStr = (String)JOptionPane.showInputDialog(getParentFrame(), 
                        "Enter seed for random number generator:", "Input",
                        JOptionPane.QUESTION_MESSAGE, null, null, "" + suggestion);
        if (seedStr != null)
            try
            {
                seed = Integer.parseInt(seedStr.trim());
                return true;
            } 
            catch (Exception ex) { Toolkit.getDefaultToolkit().beep(); }
        return false;
    }
    
}
