/*
 * File: GLabel.java
 * -----------------
 * This class defines a graphics object whose visual representation is
 * a text string, similar to the Label class in java.awt.
 */

// ************************************************************************
// *  For the beta release, the source code for the libraries continues   *
// *  to be licensed under the following Creative Commons license:        *
// *                                                                      *
// *                 Attribution-NonCommercial-NoDerivs                   *
// *                                                                      *
// *  When the final release is posted, we plan to reduce these           *
// *  restrictions to allow any use, protecting only the ACM's right      *
// *  to prevent others from selling these libraries as their own.        *
// ************************************************************************

package acm.graphics;

import acm.util.*;

import java.awt.*;

/* Class: GLabel */
/**
 * The <code>GLabel</code> class is a graphical object whose appearance consists
 * of a text string.
 *
 * <p><table cellpadding=0 border=0>
 * <tr>
 * <td width=560 valign=top>
 * To use the <code>GLabel</code> class, the first
 * step is to construct a new <code>GLabel</code>
 * object and add it to an existing <code>GCanvas</code> (assumed here to be
 * stored in the variable <code>gc</code>), as follows:
 *
 * <p><dd>
 * <code>
 * GLabel glabel = new GLabel("Hello, world.", 50, 60);<br>
 * gc.add(glabel);<br>
 * </code>
 * </dd><p>
 *
 * This code creates a <code>GLabel</code> containing
 * the string <code>"Hello, world."</code> with its origin
 * at the point (50,&nbsp;60) and installs the label in the
 * <code>GCanvas</code> as shown in the figure to the right.
 * </td>
 * <td width=230 align=right valign=top>
 * <img src="../../../../images/GLabelTest1.gif">
 * </td>
 * </tr>
 * </table>
 *
 * <p><table cellpadding=0 border=0>
 * <tr>
 * <td width=560 valign=top>
 * Most graphical objects in Java use the upper left corner of the
 * figure to define their location.  Strings, however, work differently.
 * The location at which a string is displayed is always taken to be
 * at the leftmost edge of the first character along what is called
 * the <b>baseline</b>, which is the line on which the uppercase letters
 * sit.  Some lowercase letters (<code>g</code>, <code>j</code>, <code>p</code>,
 * <code>q</code>, and <code>y</code>) descend below the baseline, as do
 * several special characters like the comma.  The location of the start
 * of the string and the concept of the baseline are illustrated in
 * the diagram to the right.
 * </td>
 * <td width=230 align=right valign=top>
 * <img src="../../../../images/GLabelTest2.gif">
 * </td>
 * </tr>
 * </table>
 *
 * <p><table cellpadding=0 border=0>
 * <tr>
 * <td width=560 valign=top>
 * Unless you specify otherwise, a string is displayed in a standard
 * font, which is defined in this class as the constant
 * <a href="#DEFAULT_FONT"><code>DEFAULT_FONT</code></a>.
 * You can change the appearance, style, and size of the string by
 * using the <a href="#setFont(Font)"><code>setFont</code></a> method,
 * which takes a <code>Font</code> as its argument, as defined in the
 * <code>java.awt</code> package.  For example, to change the string
 * so that it uses a boldface serif type face at a larger point size,
 * you could execute the following code:
 *
 * <p><dd>
 * <code>
 * glabel.setFont(new Font("Serif", Font.BOLD, 18));<br>
 * </code>
 * </dd><p>
 *
 * Note that changing the font can change the displayed size of the string,
 * which will be reflected in calls to <a href="#getWidth()"><code>getWidth</code></a>
 * or <a href="#getSize()"><code>getSize</code></a>.
 * </td>
 * <td width=230 align=right valign=top>
 * <img src="../../../../images/GLabelTest3.gif">
 * </td>
 * </tr>
 * </table>
 *
 * <p><table cellpadding=0 border=0>
 * <tr>
 * <td width=560 valign=top>
 * Just as with other objects of the
 * <a href="GObject.html"><code>GObject</code></a> class,
 * you can change the color of the displayed text by invoking
 * the <a href="#setColor(Color)"><code>setColor</code></a> method,
 * which takes a <code>Color</code> as its argument, as defined in the
 * <code>java.awt</code> package.  Thus, the following statement
 * changes the text color to blue:
 *
 * <p><dd>
 * <code>
 * glabel.setColor(Color.blue);<br>
 * </code>
 * </dd>
 * </td>
 * <td width=230 align=right valign=top>
 * <img src="../../../../images/GLabelTest4.gif">
 * </td>
 * </tr>
 * </table>
 *
 * <p><table cellpadding=0 border=0>
 * <tr>
 * <td width=560 valign=top>
 * Once installed on the canvas, you can move a string using either
 * the <a href="#setLocation(double, double)"><code>setLocation</code></a> method,
 * which moves an object to a specific position,
 * or the <a href="#move(double, double)"><code>move</code></a> method,
 * which moves an object relative to its current position.
 * To move the string rightward 50 pixels, for example, you could use
 * the following statement:
 *
 * <p><dd>
 * <code>
 * glabel.move(50, 0);<br>
 * </code>
 * </dd><p>
 *
 * As you can see from the diagram, the string is clipped by the edges of the canvas.
 * </td>
 * <td width=230 align=right valign=top>
 * <img src="../../../../images/GLabelTest5.gif">
 * </td>
 * </tr>
 * </table>
 *
 * <p><table cellpadding=0 border=0>
 * <tr>
 * <td width=560 valign=top>
 * For a <code>GLabel</code> object, one of the operations you
 * will often want to perform is to center the string relative to a particular
 * position, such as the center of the screen.  To do so, the simplest
 * approach is to use the <a href="#getWidth()"><code>getWidth</code></a>
 * method to find the width of the string in pixels, and then start the
 * string half that distance to the left of the central point.  Thus, the
 * following <a href="#setLocation(double, double)"><code>setLocation</code></a> call
 * centers the string horizontally in the <code>GCanvas</code>:
 *
 * <p><dd>
 * <code>
 * glabel.setLocation((gc.getWidth() - glabel.getWidth()) / 2, 60);<br>
 * </code>
 * </dd>
 * </td>
 * <td width=230 align=right valign=top>
 * <img src="../../../../images/GLabelTest6.gif">
 * </td>
 * </tr>
 * </table>
 *
 * <p><table cellpadding=0 border=0>
 * <tr>
 * <td width=560 valign=top>
 * As a final example, you can use the
 * <a href="#setLabel(String)"><code>setLabel</code></a> method to
 * change the character string that the <code>GLabel</code> displays.
 * Thus, if you call
 *
 * <p><dd>
 * <code>
 * glabel.setLabel("Hi!");<br>
 * </code>
 * </dd><p>
 *
 * the string <code>"Hello,&nbsp;world."</code> gets replaced by
 * <code>"Hi!"</code>.  The position of the string doesn&#146;t change,
 * so that you would then need to repeat the centering operation
 * if you wanted this string to appear in the middle of the window.
 * </td>
 * <td width=230 align=right valign=top>
 * <img src="../../../../images/GLabelTest7.gif">
 * </td>
 * </tr>
 * </table>
 */
public class GLabel extends GObject {

/* Constant: DEFAULT_FONT */
/**
 * The default font used to display strings.  You can change the font by invoking
 * the <a href="#setFont(Font)"><code>setFont</code></a> method.
 */
	public static final Font DEFAULT_FONT = new Font("Default", Font.PLAIN, 12);

/* Constructor: GLabel(str, x, y) */
/**
 * Creates a new <code>GLabel</code> object initialized to contain the specified string.
 *
 * @usage GLabel glabel = new GLabel(str);
 * @param str The initial contents of the <code>GLabel</code>
 */
	public GLabel(String str) {
		this(str, 0, 0);
	}

/* Constructor: GLabel(str, x, y) */
/**
 * Creates a new <code>GLabel</code> object with its baseline origin at the
 * specified position.
 *
 * @usage GLabel glabel = new GLabel(str, x, y);
 * @param str The initial contents of the <code>GLabel</code>
 * @param x The x-coordinate of the label origin
 * @param y The y-coordinate of the baseline for the label
 */
	public GLabel(String str, double x, double y) {
		label = str;
		setFont(DEFAULT_FONT);
		setLocation(x, y);
	}

/* Method: setFont(font) */
/**
 * Changes the font used to display the <code>GLabel</code>.  This call will
 * usually change the size of the displayed object and will therefore affect
 * the result of calls to <a href="GObject.html#getSize()"><code>getSize</code></a>
 * and <a href="GObject.html#getBounds()"><code>getBounds</code></a>.
 *
 * @usage glabel.setFont(font);
 * @param font A <code>Font</code> object indicating the new font
 */
	public void setFont(Font font) {
		labelFont = font;
		repaint();
	}

/* Method: setFont(str) */
/**
 * Changes the font used to display the <code>GLabel</code> as specified by
 * the string <code>str</code>, which is interpreted in the style of
 * <code>Font.decode</code>.  The usual format of the font string is
 *
 * <p>&nbsp;&nbsp;&nbsp;<i>family</i>code>-</code></i>style<i>code>-</code><i>size</i>
 *
 * where both <i>style</i> and <i>size</i> are optional.  If any of these parts
 * are specified as an asterisk, the existing value is retained.
 *
 * @usage glabel.setFont(str);
 * @param str A <code>String</code> specifying the new font
 */
	public void setFont(String str) {
		setFont(JTFTools.decodeFont(str, getFont()));
	}

/* Method: getFont() */
/**
 * Returns the font in which the <code>GLabel</code> is displayed.
 *
 * @usage Font font = glabel.getFont();
 * @return The font in use by this object
 */
	public Font getFont() {
		return labelFont;
	}

/* Method: setLabel(str) */
/**
 * Changes the string stored within the <code>GLabel</code> object, so that
 * a new text string appears on the display.
 *
 * @usage glabel.setLabel(str);
 * @param str The new string to display
 */
	public void setLabel(String str) {
		label = str;
		repaint();
	}

/* Method: getLabel() */
/**
 * Returns the string displayed by this object.
 *
 * @usage String str = glabel.getLabel();
 * @return The string displayed by this object
 */
	public String getLabel() {
		return label;
	}

/* Method: paint(g) */
/**
 * Implements the <code>paint</code> operation for this graphical object.  This method
 * is not called directly by clients.
 * @noshow
 */
	public void paint(Graphics g) {
		g.setFont(labelFont);
		g.drawString(label, GMath.round(getX()), GMath.round(getY()));
	}

/* Method: getWidth() */
/**
 * Returns the width of this string, as it appears on the display.
 *
 * @usage double width = glabel.getWidth();
 * @return The width of this object
 */
	public double getWidth() {
		return getFontMetrics().stringWidth(label);
	}

/* Method: getHeight() */
/**
 * Returns the height of this string, as it appears on the display.
 *
 * @usage double height = glabel.getHeight();
 * @return The height of this string
 */
	public double getHeight() {
		return getFontMetrics().getHeight();
	}

/* Method: getAscent() */
/**
 * Returns the distance this string extends above the baseline.
 *
 * @usage double ascent = glabel.getAscent();
 * @return The ascent of this string in pixels
 */
	public double getAscent() {
		return getFontMetrics().getAscent();
	}

/* Method: getDescent() */
/**
 * Returns the distance this string descends below the baseline.
 *
 * @usage double descent = glabel.getDescent();
 * @return The descent of this string in pixels
 */
	public double getDescent() {
		return getFontMetrics().getDescent();
	}

/* Method: getFontMetrics() */
/**
 * Returns a <code>FontMetrics</code> object describing the dimensions of this string.
 *
 * @usage FontMetrics fm = glabel.getFontMetrics();
 * @return A <code>FontMetrics</code> object describing the dimensions of this string
 * @noshow
 */
	public FontMetrics getFontMetrics() {
		Component comp = getComponent();
		if (comp == null) comp = DUMMY_COMPONENT;
		return comp.getFontMetrics(labelFont);
	}

/* Method: getBounds() */
/**
 * Returns a <code>GRectangle</code> that specifies the bounding box for the string.
 *
 * @usage GRectangle bounds = glabel.getBounds();
 * @return The bounding box for this object
 */
	public GRectangle getBounds() {
		return new GRectangle(getX(), getY() - getAscent(), getWidth(), getHeight());
	}

/* Inherited method: getSize() */
/**
 * @inherited GObject#GDimension getSize()
 * Returns the size of the bounding box for this object.
 */

/* Inherited method: contains(x, y) */
/**
 * @inherited GObject#boolean contains(double x, double y)
 * Checks to see whether a point is "inside" the string, which is defined to be
 * inside the bounding rectangle.
 */

/* Inherited method: contains(pt) */
/**
 * @inherited GObject#boolean contains(GPoint pt)
 * Checks to see whether a point is inside the object.
 */

/* Inherited method: setLocation(x, y) */
/**
 * @inherited GObject#void setLocation(double x, double y)
 * Sets the location of the <code>GLabel</code> to the point (<code>x</code>, <code>y</code>).
 * For a <code>GLabel</code>, the location is the point on the text baseline at which the
 * text starts.
 */

/* Inherited method: setLocation(pt) */
/**
 * @inherited GObject#void setLocation(GPoint pt)
 * Sets the location of this object to the specified point.
 */

/* Inherited method: getLocation() */
/**
 * @inherited GObject#GPoint getLocation()
 * Returns the location of the <code>GLabel</code> as a <code>GPoint</code> object.
 */

/* Inherited method: getX() */
/**
 * @inherited GObject#double getX()
 * Returns the x-coordinate of the object.
 */

/* Inherited method: getY() */
/**
 * @inherited GObject#double getY()
 * Returns the y-coordinate of the object.
 */

/* Inherited method: move(dx, dy) */
/**
 * @inherited GObject#void move(double dx, double dy)
 * Moves the object on the screen using the displacements <code>dx</code> and <code>dy</code>.
 */

/* Inherited method: movePolar(r, theta) */
/**
 * @inherited GObject#void movePolar(double r, double theta)
 * Moves the object using displacements given in polar coordinates.
 */

/* Inherited method: sendToFront() */
/**
 * @inherited GObject#void sendToFront()
 * Moves this object to the front of the display in the <i>z</i> dimension.
 */

/* Inherited method: sendToBack() */
/**
 * @inherited GObject#void sendToBack()
 * Moves this object to the back of the display in the <i>z</i> dimension.
 */

/* Inherited method: sendForward() */
/**
 * @inherited GObject#void sendForward()
 * Moves this object one step toward the front in the <i>z</i> dimension.
 */

/* Inherited method: sendBackward() */
/**
 * @inherited GObject#void sendBackward()
 * Moves this object one step toward the back in the <i>z</i> dimension.
 */

/* Inherited method: setColor(color) */
/**
 * @inherited GObject#void setColor(Color color)
 * Sets the color used to display the text of the <code>GLabel</code>.
 */

/* Inherited method: getColor() */
/**
 * @inherited GObject#Color getColor()
 * Returns the color used to display the text of the <code>GLabel</code>.
 */

/* Inherited method: setVisible(visible) */
/**
 * @inherited GObject#void setVisible(boolean visible)
 * Sets the visibility status of the <code>GLabel</code>.
 */

/* Inherited method: isVisible() */
/**
 * @inherited GObject#boolean isVisible()
 * Checks to see whether the <code>GLabel</code> is visible.
 */

/* Inherited method: addMouseListener(listener) */
/**
 * @inherited GObject#void addMouseListener(MouseListener listener)
 * Adds a mouse listener to this graphical object.
 */

/* Inherited method: removeMouseListener(listener) */
/**
 * @inherited GObject#void removeMouseListener(MouseListener listener)
 * Removes a mouse listener from this graphical object.
 */

/* Inherited method: addMouseMotionListener(listener) */
/**
 * @inherited GObject#void addMouseMotionListener(MouseMotionListener listener)
 * Adds a mouse motion listener to this graphical object.
 */

/* Inherited method: removeMouseMotionListener(listener) */
/**
 * @inherited GObject#void removeMouseMotionListener(MouseMotionListener listener)
 * Removes a mouse motion listener from this graphical object.
 */

/* Protected method: paramString() */
/**
 * Returns a string indicating the parameters of this object.
 * @noshow
 */
	public String paramString() {
		return super.paramString() + ", string=\"" + label + "\"";
	}

/* Private state */

	private String label;
	private Font labelFont;

	private static final Component DUMMY_COMPONENT = MediaTools.getImageObserver();
}
