/*
 * File: ProgramMenuBar.java
 * -------------------------
 * This file implements the standard menu bar for programs.
 */

// ************************************************************************
// *  For the beta release, the source code for the libraries continues   *
// *  to be licensed under the following Creative Commons license:        *
// *                                                                      *
// *                 Attribution-NonCommercial-NoDerivs                   *
// *                                                                      *
// *  When the final release is posted, we plan to reduce these           *
// *  restrictions to allow any use, protecting only the ACM's right      *
// *  to prevent others from selling these libraries as their own.        *
// ************************************************************************

package acm.program;

import acm.io.*;
import acm.util.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;

/* Class: ProgramMenuBar */
/**
 * This class standardizes the menu bars used in the ACM program package.
 */
public class ProgramMenuBar extends JMenuBar {

/* Constructor: ProgramMenuBar(program) */
/**
 * Creates a <code>ProgramMenuBar</code> with no menus.
 *
 * @usage ProgramMenuBar mbar = new ProgramMenuBar(program);
 */
	public ProgramMenuBar(Program program) {
		myProgram = program;
		listener = new ProgramMenuListener(program);
		init();
	}

/* Method: setConsole(console) */
/**
 * Sets the console associated with the menu bar.  Each console calls this method
 * whenever it acquires the keyboard focus, which allows the menu bar to work with
 * multiple consoles.
 *
 * @usage mbar.setConsole(console);
 * @param console The <code>IOConsole</code> object to be used with the menu bar
 */
	public void setConsole(IOConsole console) {
		activeConsole = console;
	}

/* Method: getConsole() */
/**
 * Returns the console associated with this menu bar.
 *
 * @usage IOConsole console = mbar.getConsole();
 * @return The <code>IOConsole</code> object currently associated with the menu bar
 */
	public IOConsole getConsole() {
		return activeConsole;
	}

/* Method: update() */
/**
 * Updates the state of each interactor in the menu bar by calling back to the
 * <code>updateMenuItem</code> method in the program.
 *
 * @usage mbar.getConsole();
 * @return The <code>IOConsole</code> object currently associated with the menu bar
 */
 	public void update() {
		int nMenus = getMenuCount();
		for (int i = 0; i < nMenus; i++) {
			updateMenuItem(getMenu(i));
		}
	}

/* Factory method: init() */
/**
 * Initializes the menu bar.  Subclasses must override this method
 * with an implementation that adds the desired menus.
 *
 * @usage init();
 */
	protected void init() {
		add(createFileMenu());
		add(createEditMenu());
	}

/* Factory method: createFileMenu() */
/**
 * Creates the <code>File</code> menu.  The standard implementation creates
 * a minimal menu that contains only a <code>Quit</code> item.
 *
 * @usage JMenu menu = mbar.createFileMenu();
 * @return The new <code>File</code> menu
 */
	protected JMenu createFileMenu() {
		JMenu menu = new JMenu("File");
		menu.setMnemonic(KeyEvent.VK_F);
		addSaveItems(menu);
		menu.addSeparator();
		addPrintItems(menu);
		menu.addSeparator();
		addQuitItem(menu);
		return menu;
	}

/* Factory method: createEditMenu() */
/**
 * Creates the <code>Edit</code> menu.  The standard implementation creates
 * a minimal menu that contains only the standard cut/copy/paste items.
 *
 * @usage JMenu menu = mbar.createEditMenu();
 * @return The new <code>Edit</code> menu
 */
	protected JMenu createEditMenu() {
		JMenu menu = new JMenu("Edit");
		menu.setMnemonic(KeyEvent.VK_E);
		addEditItems(menu);
		return menu;
	}

/* Protected method: addQuitItem(menu) */
/**
 * Adds a <code>Quit</code> item to the menu, which is called <code>Quit</code>
 * on a Macintosh and <code>Exit</code> on other platforms.  The action command
 * generated is always <code>"Quit"</code>.
 *
 * @usage addQuitItem(menu);
 */
	protected void addQuitItem(JMenu menu) {
		if (Platform.isMac()) {
			JMenuItem item = createMenuItem("Quit", KeyEvent.VK_Q, 0);
			item.setEnabled(true);
			menu.add(item);
		} else {
			JMenuItem item = createMenuItem("Exit", 0, KeyEvent.VK_X);
			item.setActionCommand("Quit");
			item.setEnabled(true);
			menu.add(item);
		}
	}

/* Protected method: addEditItems(menu) */
/**
 * Adds the standard Edit menu entries to the menu.  The action commands
 * generated are always <code>"Cut"</code>, <code>"Copy"</code>,
 * <code>"Paste"</code>, and <code>"Select All"</code>.
 *
 * @usage mbar.addEditItems(menu);
 */
	protected void addEditItems(JMenu menu) {
		if (Platform.isMac()) {
			menu.add(createMenuItem("Cut", KeyEvent.VK_X, 0));
			menu.add(createMenuItem("Copy", KeyEvent.VK_C, 0));
			menu.add(createMenuItem("Paste", KeyEvent.VK_V, 0));
			menu.add(createMenuItem("Select All", KeyEvent.VK_A, 0));
		} else {
			JMenuItem item = createMenuItem("Cut (x)", KeyEvent.VK_X, 0);
			item.setActionCommand("Cut");
			menu.add(item);
			item = createMenuItem("Copy (c)", KeyEvent.VK_C, 0);
			item.setActionCommand("Copy");
			menu.add(item);
			item = createMenuItem("Paste (v)", KeyEvent.VK_V, 0);
			item.setActionCommand("Paste");
			menu.add(item);
			menu.add(createMenuItem("Select All", KeyEvent.VK_A, 0));
		}
	}

/* Protected method: addSaveItems(menu) */
/**
 * Adds the standard Save items to the menu.
 *
 * @usage mbar.addSaveItems(menu);
 */
	protected void addSaveItems(JMenu menu) {
		menu.add(createMenuItem("Save", KeyEvent.VK_S, 0));
		menu.add(createMenuItem("Save As...", 0, 0));
	}

/* Protected method: addPrintItems(menu) */
/**
 * Adds the standard Print/Script items to the menu.
 *
 * @usage mbar.addPrintItems(menu);
 */
	protected void addPrintItems(JMenu menu) {
		menu.add(createMenuItem("Print", KeyEvent.VK_P, 0));
		menu.add(createMenuItem("Print Console", 0, 0));
		menu.add(createMenuItem("Script...", 0, 0));
	}

/* Protected method: createProgramMenu */
/**
 * Creates a single <code>Menu</code> that contains all of the interactors
 * for the menu bar.
 */
	protected Menu createProgramMenu() {
		Menu programMenu = new Menu("Program");
		int nMenus = getMenuCount();
		for (int i = 0; i < nMenus; i++) {
			addProgramMenuItems(programMenu, getMenu(i));
		}
		return programMenu;
	}

/* Protected method: createMacMenuBar */
/**
 * Creates a <code>MenuBar</code> that has the same effect as the
 * specified <code>JMenuBar</code>.
 */
	protected MenuBar createMacMenuBar() {
		MenuBar mbar = new MenuBar();
		int nMenus = getMenuCount();
		for (int i = 0; i < nMenus; i++) {
			mbar.add(createMacMenu(getMenu(i)));
		}
		return mbar;
	}

/* Private method: createMenuItem(text, accelerator, mnemonic) */
/**
 * Creates a menu item with the specified text, accelerator, and mnemonic.
 *
 * @usage JMenuItem item = mbar.createMenuItem(item, accelerator, mnemonic);
 * @param text The text displayed in the menu item label
 * @param accelerator A key code (from <code>KeyEvent</code>) that specifies the accelerator
 * @param mnemonic A key code (from <code>KeyEvent</code>) that specifies the mnemonic
 */
	private JMenuItem createMenuItem(String text, int accelerator, int mnemonic) {
		JMenuItem item = (mnemonic == 0) ? new JMenuItem(text) : new JMenuItem(text, mnemonic);
		item.addActionListener(listener);
		if (accelerator != 0) {
			item.setAccelerator(KeyStroke.getKeyStroke((char) accelerator, KeyEvent.CTRL_MASK));
		}
		item.setEnabled(false);
		if (text.endsWith("...")) item.setActionCommand(text.substring(0, text.length() - 3));
		return item;
	}

/* Private method: createMacMenu */
/**
 * Creates a <code>Menu</code> that has the same effect as the
 * specified <code>JMenu</code>.
 */
	private Menu createMacMenu(JMenu jmenu) {
		Menu menu = new Menu(jmenu.getText());
		int nItems = jmenu.getItemCount();
		for (int i = 0; i < nItems; i++) {
			menu.add(createMacMenuItem((JMenuItem) jmenu.getItem(i)));
		}
		return menu;
	}

/* Private method: createMacMenuItem */
/**
 * Creates a <code>MenuItem</code> that has the same effect as the
 * specified <code>JMenuItem</code>.
 */
	private MenuItem createMacMenuItem(JMenuItem jitem) {
		if (jitem == null) {
			return new MenuItem("-");
		} else if (jitem instanceof JMenu) {
			return createMacMenu((JMenu) jitem);
		} else if (jitem instanceof JCheckBoxMenuItem) {
			return new MacCheckboxMenuItem((JCheckBoxMenuItem) jitem);
		} else {
			return new MacMenuItem(jitem);
		}
	}

/* Private method: addProgramMenuItems(programMenu, item) */
/**
 * Adds the item to the program menu, recursively if item is a submenu.
 */
	private void addProgramMenuItems(Menu programMenu, JMenuItem jitem) {
		if (jitem == null) {
			programMenu.addSeparator();
		} else if (jitem instanceof JMenu) {
			JMenu jmenu = (JMenu) jitem;
			int nItems = jmenu.getItemCount();
			for (int i = 0; i < nItems; i++) {
				addProgramMenuItems(programMenu, jmenu.getItem(i));
			}
		} else if (jitem instanceof JCheckBoxMenuItem) {
			programMenu.add(new MacCheckboxMenuItem((JCheckBoxMenuItem) jitem));
		} else {
			if (!jitem.getActionCommand().equals("Quit")) {
				programMenu.add(new MacMenuItem(jitem));
			}
		}
	}

/* Private method: updateMenuItem(item) */
/**
 * Updates the enabled state of the specified menu item.
 *
 * @usage updateMenuItem(item);
 * @param item The <code>JMenuItem</code> being updated
 */
	protected void updateMenuItem(JMenuItem item) {
		if (item instanceof JMenu) {
			JMenu menu = (JMenu) item;
			int nItems = menu.getItemCount();
			for (int i = 0; i < nItems; i++) {
				JMenuItem subItem = menu.getItem(i);
				if (subItem != null) updateMenuItem(subItem);
			}
		} else {
			myProgram.updateMenuItem(item);
		}
	}

/* Private instance variables */

	private Program myProgram;
	private ProgramMenuListener listener;
	private IOConsole activeConsole;
}

/* Package class: ProgramMenuListener */
/**
 * This class implements the listener for the standard menu items that
 * forwards their action back to the program.
 */
class ProgramMenuListener implements ActionListener {

/* Constructor: ProgramMenuListener(program) */
/**
 * Creates a new listener for the standard menu items that will be added to this
 * menu bar.
 */
	public ProgramMenuListener(Program program) {
		myProgram = program;
	}

/* Method: actionPerformed(e) */
/**
 * Responds to an action event in the corresponding menu.   The effect of an
 * action event is to forward the action command back to the program.
 */
	public void actionPerformed(ActionEvent e) {
		myProgram.menuAction(e.getActionCommand());
	}

/* Private instance variables */

	private Program myProgram;

}

/* Package class: MacMenuItem */
/**
 * This class represents a standard Macintosh <code>MenuItem</code> that listens to
 * a <code>JMenuItem</code> and tracks its changes.
 */
class MacMenuItem extends MenuItem implements ActionListener, ChangeListener {

/* Constructor: MacMenuItem(jitem) */
/**
 * Creates a new <code>MenuItem</code> that tracks the changes in the specified
 * <code>JMenuItem</code>.
 */
	public MacMenuItem(JMenuItem jitem) {
		super(jitem.getText());
		twin = jitem;
		addActionListener(this);
		twin.addChangeListener(this);
		setEnabled(twin.isEnabled());
		KeyStroke accelerator = twin.getAccelerator();
		if (accelerator != null) setShortcut(createShortcut(accelerator));
	}

/* Method: actionPerformed(e) */
/**
 * Responds to an action event in the Mac menu and forwards it along to
 * the actual <code>JMenuItem</code> that the client has created.
 */
	public void actionPerformed(ActionEvent e) {
		if (e != e) /* Avoid the unused parameter warning */;
		twin.doClick(0);
	}

/* Method: stateChanged(e) */
/**
 * Monitors the state of the <code>JMenuItem</code> and replicates changes
 * in the enabled state.
 */
	public void stateChanged(ChangeEvent e) {
		setEnabled(twin.isEnabled());
	}

/* Private method: createShortcut(accelerator) */
/**
 * Creates an old-style menu shortcut from the new-style accelerator.
 */
	private MenuShortcut createShortcut(KeyStroke accelerator) {
		boolean isShifted = (accelerator.getModifiers() & Event.SHIFT_MASK) != 0;
		return new MenuShortcut(accelerator.getKeyCode(), isShifted);
	}

/* Private instance variables */

	private JMenuItem twin;
}

/* Package class: MacCheckboxMenuItem */
/**
 * This class represents a standard Macintosh <code>CheckboxMenuItem</code> that
 * listens to a <code>JCheckBoxMenuItem</code> and tracks its changes.
 */
class MacCheckboxMenuItem extends CheckboxMenuItem implements ActionListener, ChangeListener {

/* Constructor: MacCheckboxMenuItem(jitem) */
/**
 * Creates a new <code>CheckboxMenuItem</code> that tracks the changes in the specified
 * <code>JCheckBoxMenuItem</code>.
 */
	public MacCheckboxMenuItem(JCheckBoxMenuItem jitem) {
		super(jitem.getText());
		twin = jitem;
		addActionListener(this);
		twin.addChangeListener(this);
		setState(twin.getState());
		setEnabled(twin.isEnabled());
		KeyStroke accelerator = twin.getAccelerator();
		if (accelerator != null) setShortcut(createShortcut(accelerator));
	}

/* Method: actionPerformed(e) */
/**
 * Responds to an action event in the Mac menu and forwards it along to
 * the actual <code>JMenuItem</code> that the client has created.
 */
	public void actionPerformed(ActionEvent e) {
		if (e != e) /* Avoid the unused parameter warning */;
		twin.doClick(0);
	}

/* Method: stateChanged(e) */
/**
 * Monitors the state of the <code>JMenuItem</code> and replicates changes
 * in the enabled state.
 */
	public void stateChanged(ChangeEvent e) {
		setState(twin.getState());
		setEnabled(twin.isEnabled());
	}

/* Private method: createShortcut(accelerator) */
/**
 * Creates an old-style menu shortcut from the new-style accelerator.
 */
	private MenuShortcut createShortcut(KeyStroke accelerator) {
		boolean isShifted = (accelerator.getModifiers() & Event.SHIFT_MASK) != 0;
		return new MenuShortcut(accelerator.getKeyCode(), isShifted);
	}

/* Private instance variables */

	private JCheckBoxMenuItem twin;
}
