/*
 * File: JTFTools.java
 * -------------------
 * This class provides a small collection of static utility methods
 * that are used elsewhere in the ACM packages.
 */

// ************************************************************************
// *  For the beta release, the source code for the libraries continues   *
// *  to be licensed under the following Creative Commons license:        *
// *                                                                      *
// *                 Attribution-NonCommercial-NoDerivs                   *
// *                                                                      *
// *  When the final release is posted, we plan to reduce these           *
// *  restrictions to allow any use, protecting only the ACM's right      *
// *  to prevent others from selling these libraries as their own.        *
// ************************************************************************

package acm.util;

import java.applet.*;
import java.awt.*;
import java.util.*;

/* Class: JTFTools */
/**
 * This class provides a small collection of static utility methods
 * that are used elsewhere in the ACM packages.
 */

public class JTFTools {

/* Static method: createEmptyContainer() */
/**
 * Returns an empty lightweight container.  Several packages need to create
 * such components as placeholders.  Defining it in JTFTools gives those
 * pacakges access to a common mechanism.
 *
 * @usage Container comp = JTFTools.createEmptyContainer();
 * @return An empty lightweight container that can be used as a placeholder
 */
	public static Container createEmptyContainer() {
		return new EmptyContainer();
	}

/* Static method: getEnclosingFrame(comp) */
/**
 * Returns the frame that encloses the specified component.
 *
 * @usage Frame frame = JTFTools.getEnclosingFrame(comp);
 * @param comp The component at which to start the search
 * @return The nearest enclosing <code>Frame</code> object
 */
	public static Frame getEnclosingFrame(Component comp) {
		while (comp != null && !(comp instanceof Frame)) {
			comp = comp.getParent();
		}
		return (Frame) comp;
	}

/* Static method: decodeFont(str) */
/**
 * Decodes a font in the style of <code>Font.decode</code>.  This version of
 * the method is included to centralize font operations and is functionally
 * identical to <code>Font.decode</code>.
 *
 * @usage Font font = JTFTools.decodeFont(str);
 * @param str The string to decode
 * @return The new font
 */
	public static Font decodeFont(String str) {
		return decodeFont(str, null);
	}

/* Static method: decodeFont(str, oldFont) */
/**
 * Decodes a font in the style of <code>Font.decode</code>.  The only difference
 * is that this method takes a font parameter that gives default values for
 * the different parts of the font.  If the family, size, or style is specified
 * as an asterisk, the corresponding value is taken from the supplied font.
 *
 * @usage Font font = JTFTools.decodeFont(str, oldFont);
 * @param str The string to decode
 * @param oldFont The font whose properties are used as defaults
 * @return The new font
 */
	public static Font decodeFont(String str, Font oldFont) {
		String fontName = str;
		int fontStyle = Font.PLAIN;
		int fontSize = 12;
		int hyphen = str.indexOf('-');
		if (hyphen >= 0) {
			fontName = str.substring(0, hyphen);
			str = str.substring(hyphen + 1);
			hyphen = str.indexOf('-');
			if (hyphen >= 0) {
				String token = str.substring(0, hyphen).toLowerCase();
				if (token.equals("*") && oldFont != null) {
					fontStyle = oldFont.getStyle();
				} else if (token.equals("plain")) {
					fontStyle = Font.PLAIN;
				} else if (token.equals("bold")) {
					fontStyle = Font.BOLD;
				} else if (token.equals("italic")) {
					fontStyle = Font.ITALIC;
				} else if (token.equals("bolditalic")) {
					fontStyle = Font.BOLD | Font.ITALIC;
				} else {
					throw new ErrorException("Illegal font style");
				}
				str = str.substring(hyphen + 1);
			}
			if (str.equals("*") && oldFont != null) {
				fontSize = oldFont.getSize();
			} else {
				try {
					fontSize = Integer.valueOf(str).intValue();
				} catch (NumberFormatException ex) {
					throw new ErrorException("Illegal font size");
				}
			}
		}
		if (fontName.equals("*")) {
			fontName = (oldFont == null) ? "Default" : oldFont.getName();
		}
		return new Font(fontName, fontStyle, fontSize);
	}

/* Static method: decodeColor(name) */
/**
 * Decodes a color name.  This method is similar to <code>Color.decode</code>
 * except in that it allows named colors and system colors.
 *
 * @usage color = JTFTools.decodeColor(name);
 * @param name The string name of the color
 * @return The color corresponding to the specified name
 */
	public static Color decodeColor(String name) {
		if (name.equalsIgnoreCase("desktop")) return SystemColor.desktop;
		if (name.equalsIgnoreCase("activeCaption")) return SystemColor.activeCaption;
		if (name.equalsIgnoreCase("activeCaptionText")) return SystemColor.activeCaptionText;
		if (name.equalsIgnoreCase("activeCaptionBorder")) return SystemColor.activeCaptionBorder;
		if (name.equalsIgnoreCase("inactiveCaption")) return SystemColor.inactiveCaption;
		if (name.equalsIgnoreCase("inactiveCaptionText")) return SystemColor.inactiveCaptionText;
		if (name.equalsIgnoreCase("inactiveCaptionBorder")) return SystemColor.inactiveCaptionBorder;
		if (name.equalsIgnoreCase("window")) return SystemColor.window;
		if (name.equalsIgnoreCase("windowBorder")) return SystemColor.windowBorder;
		if (name.equalsIgnoreCase("windowText")) return SystemColor.windowText;
		if (name.equalsIgnoreCase("menu")) return SystemColor.menu;
		if (name.equalsIgnoreCase("menuText")) return SystemColor.menuText;
		if (name.equalsIgnoreCase("text")) return SystemColor.text;
		if (name.equalsIgnoreCase("textText")) return SystemColor.textText;
		if (name.equalsIgnoreCase("textHighlight")) return SystemColor.textHighlight;
		if (name.equalsIgnoreCase("textHighlightText")) return SystemColor.textHighlightText;
		if (name.equalsIgnoreCase("textInactiveText")) return SystemColor.textInactiveText;
		if (name.equalsIgnoreCase("control")) return SystemColor.control;
		if (name.equalsIgnoreCase("controlText")) return SystemColor.controlText;
		if (name.equalsIgnoreCase("controlHighlight")) return SystemColor.controlHighlight;
		if (name.equalsIgnoreCase("controlLtHighlight")) return SystemColor.controlLtHighlight;
		if (name.equalsIgnoreCase("controlShadow")) return SystemColor.controlShadow;
		if (name.equalsIgnoreCase("controlDkShadow")) return SystemColor.controlDkShadow;
		if (name.equalsIgnoreCase("scrollbar")) return SystemColor.scrollbar;
		if (name.equalsIgnoreCase("info")) return SystemColor.info;
		if (name.equalsIgnoreCase("infoText")) return SystemColor.infoText;
		if (name.equalsIgnoreCase("black")) return Color.black;
		if (name.equalsIgnoreCase("blue")) return Color.blue;
		if (name.equalsIgnoreCase("cyan")) return Color.cyan;
		if (name.equalsIgnoreCase("darkGray")) return Color.darkGray;
		if (name.equalsIgnoreCase("DARK_GRAY")) return Color.darkGray;
		if (name.equalsIgnoreCase("gray")) return Color.gray;
		if (name.equalsIgnoreCase("green")) return Color.green;
		if (name.equalsIgnoreCase("lightGray")) return Color.lightGray;
		if (name.equalsIgnoreCase("LIGHT_GRAY")) return Color.lightGray;
		if (name.equalsIgnoreCase("magenta")) return Color.magenta;
		if (name.equalsIgnoreCase("orange")) return Color.orange;
		if (name.equalsIgnoreCase("pink")) return Color.pink;
		if (name.equalsIgnoreCase("red")) return Color.red;
		if (name.equalsIgnoreCase("white")) return Color.white;
		if (name.equalsIgnoreCase("yellow")) return Color.yellow;
		try {
			return Color.decode(name);
		} catch (NumberFormatException ex) {
			throw new ErrorException("Illegal color value");
		}
	}		

/* Static method: registerApplet(applet) */
/**
 * Adds this applet to a table indexed by the current thread.
 *
 * @usage JTFTools.registerApplet(applet);
 * @param applet The applet being registered
 */
	public static void registerApplet(Applet applet) {
		registerApplet(applet, Thread.currentThread());
		mostRecentApplet = applet;
	}

/* Static method: registerApplet(applet, thread) */
/**
 * Adds this applet to a table indexed by the specified thread.
 *
 * @usage JTFTools.registerApplet(applet);
 * @param applet The applet being registered
 * @param thread The thread used as the key
 */
	public static void registerApplet(Applet applet, Thread thread) {
		appletTable.put(thread, applet);
	}

/* Static method: getApplet() */
/**
 * Returns the current applet.  This implementation may fail in
 * multithreaded programs.  Such programs should therefore not
 * rely on this facility.  It is appropriate to use this facility,
 * for example, during the <code>init</code> method of a program,
 * which is the only context in which it is applied in the JTF tools.
 *
 * @usage Applet applet JTFTools.getApplet();
 * @return The currently running applet
 */
	public static Applet getApplet() {
		Applet applet = (Applet) appletTable.get(Thread.currentThread());
		if (applet == null) applet = mostRecentApplet;
		return applet;
	}

/* Static method: pause(milliseconds) */
/**
 * Delays the calling thread for the specified time, which is expressed in
 * milliseconds.  Unlike <code>Thread.sleep</code>, this method never throws an
 * exception.
 *
 * @usage JTFTools.pause(milliseconds);
 * @param milliseconds The sleep time in milliseconds
 */
	public static void pause(double milliseconds) {
		try {
			int millis = (int) milliseconds;
			int nanos = (int) Math.round((milliseconds - millis) * 1000000);
			Thread.sleep(millis, nanos);
		} catch (InterruptedException ex) {
			/* Empty */
		}
	}

/* Static variables */

	private static HashMap appletTable = new HashMap();
	private static Applet mostRecentApplet = null;
}

/* Package class: EmptyContainer */
/**
 * This class represents a simple lightweight container.  The only difference
 * between EmptyContainer and Container is that Container is abstract and
 * can therefore not be instantiated.
 */

class EmptyContainer extends Container {
	public void update(Graphics g) {
		paint(g);
	}
}
