/*
 * File: OptionTable.java
 * ----------------------
 * This class parses a string into keyword/value pairs in
 * a HashMap.  It is used by the TableLayout mechanism in
 * acm.gui but is included here in light of its more general
 * applicability.
 */

// ************************************************************************
// *  For the beta release, the source code for the libraries continues   *
// *  to be licensed under the following Creative Commons license:        *
// *                                                                      *
// *                 Attribution-NonCommercial-NoDerivs                   *
// *                                                                      *
// *  When the final release is posted, we plan to reduce these           *
// *  restrictions to allow any use, protecting only the ACM's right      *
// *  to prevent others from selling these libraries as their own.        *
// ************************************************************************

package acm.util;

import java.io.*;
import java.util.*;

public class OptionTable extends HashMap {

/* Constructor: OptionTable(str) */
/**
 * Creates a new <code>OptionTable</code> and initializes it
 * from the specified string.  The options in <code>str</code>
 * are in two possible forms:
 *
 * <ol>
 * <li>key
 * <li>key=value
 * </ol>
 *
 * The first option sets the value associated with the key to
 * the empty string; the second supplies the value explicitly.
 *
 * @usage OptionTable options = new OptionTable(str);
 * @param str The option string that is parsed to initialize the table
 */
	public OptionTable(String str) {
		this(str, null);
	}

/* Constructor: OptionTable(str, keys) */
/**
 * Creates a new <code>OptionTable</code> from the specified string,
 * checking to make sure that all keys exist in the string array
 * <code>keys</code>.  If <code>keys</code> is <code>null</code>,
 * checking is disabled.
 *
 * @usage OptionTable options = new OptionTable(str, keys);
 * @param str The option string that is parsed to initialize the table
 * @param keys An array of strings indicating the legal keys
 */
	public OptionTable(String str, String[] keys) {
		try {
			StreamTokenizer tokenizer = createTokenizer(str);
			int ttype = tokenizer.nextToken();
			while (ttype != StreamTokenizer.TT_EOF) {
				if (ttype != StreamTokenizer.TT_WORD) {
					throw new ErrorException("Illegal option string: " + str);
				}
				String key = tokenizer.sval;
				if (keys != null && !keyExists(key, keys)) {
					throw new ErrorException("Unrecognized option: " + key);
				}
				ttype = tokenizer.nextToken();
				if (ttype == '=') {
					ttype = tokenizer.nextToken();
					if (ttype != StreamTokenizer.TT_WORD && ttype != '"' && ttype != '\'') {
						throw new ErrorException("Illegal option string: " + str);
					}
					put(key, tokenizer.sval);
					ttype = tokenizer.nextToken();
				} else {
					put(key, "");
				}
			}
		} catch (IOException ex) {
			throw new ErrorException("Illegal option string: " + str);
		}
	}

/* Constructor: OptionTable(map) */
/**
 * Creates a new <code>OptionTable</code> from an existing map.
 * Most clients will not need to use this method.
 *
 * @usage OptionTable options = new OptionTable(map);
 * @param map An existing key/value mapping
 */
	public OptionTable(Map map) {
		for (Iterator i = (Iterator) map.keySet(); i.hasNext(); ) {
			String key = (String) i.next();
			String value = (String) map.get(key);
			put(key, value);
		}
	}

/* Method: isSpecified(key) */
/**
 * Returns true if the key has been specified in the option table.
 *
 * @usage if (options.isSpecified(key)) . . .
 * @param key The key being checked
 * @return <code>true</code> if <code>key</code> was specified in the option string
 */
	public boolean isSpecified(String key) {
		return containsKey(key);
	}

/* Method: getOption(key) */
/**
 * Returns the value associated with <code>key</code> in the option
 * table, or <code>null</code> if no such value exists.
 *
 * @usage String value = options.getOption(key);
 * @param key The key
 * @return The corresponding option value
 */
	public String getOption(String key) {
		return getOption(key, null);
	}

/* Method: getOption(key, defValue) */
/**
 * Returns the value associated with <code>key</code> in the option
 * table or the specified default value if no such binding exists.
 *
 * @usage String value = options.getOption(key, defValue);
 * @param key The key
 * @param defValue The default to use if the key is not found
 * @return The corresponding option value
 */
	public String getOption(String key, String defValue) {
		String value = (String) get(key);
		return (value == null || value.equals("")) ? defValue : value;
	}

/* Method: getIntOption(key) */
/**
 * Returns the integer value associated with <code>key</code> in the option
 * table, or 0 if no such value exists.
 *
 * @usage int value = options.getIntOption(key);
 * @param key The key
 * @return The corresponding option value parsed as an integer
 */
	public int getIntOption(String key) {
		return getIntOption(key, 0);
	}

/* Method: getIntOption(key, defValue) */
/**
 * Returns the integer value associated with <code>key</code> in the option
 * table or the specified default value if no such binding exists.
 *
 * @usage int value = options.getIntOption(key, defValue);
 * @param key The key
 * @param defValue The default to use if the key is not found
 * @return The corresponding option value
 */
	public int getIntOption(String key, int defValue) {
		String binding = getOption(key, null);
		if (binding == null || binding.equals("")) return defValue;
		return Integer.decode(binding).intValue();
	}

/* Method: getDoubleOption(key) */
/**
 * Returns the <code>double</code> value associated with <code>key</code>
 * in the option table, or 0.0 if no such value exists.
 *
 * @usage double value = options.getDoubleOption(key);
 * @param key The key
 * @return The corresponding option value parsed as a <code>double</code>
 */
	public double getDoubleOption(String key) {
		return getDoubleOption(key, 0.0);
	}

/* Method: getDoubleOption(key, defValue) */
/**
 * Returns the <code>double</code> value associated with <code>key</code>
 * in the option table or the specified default value if no such binding
 * exists.
 *
 * @usage double value = options.getDoubleOption(key, defValue);
 * @param key The key
 * @param defValue The default to use if the key is not found
 * @return The corresponding option value
 */
	public double getDoubleOption(String key, double defValue) {
		String binding = getOption(key, null);
		if (binding == null || binding.equals("")) return defValue;
		return Double.valueOf(binding).doubleValue();
	}

/* Private method: createTokenizer(str) */
/**
 * Creates a tokenizer for the specified string.
 */
	private StreamTokenizer createTokenizer(String str) {
		StreamTokenizer t = new StreamTokenizer(new StringReader(str));
		t.resetSyntax();
		t.wordChars((char) 33, (char) ('=' - 1));
		t.wordChars((char) ('=' + 1), (char) 126);
		t.quoteChar('"');
		t.quoteChar('\'');
		t.whitespaceChars(' ', ' ');
		t.whitespaceChars('\t', '\t');
		return t;
	}

/* Private method: keyExists(key, keys) */
/**
 * Checks to see whether the key exists in the array of keys.
 */
	private boolean keyExists(String key, String[] keys) {
		for (int i = 0; i < keys.length; i++) {
			if (key.equals(keys[i])) return true;
		}
		return false;
	}
}
