/*
 * Person.java
 * 
 * Copyright (c) 2004 ICEGT, Georgia Institute of Technology, Jeremy Goecks
 */
package edu.gatech.icegt.guesswho;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

/**
 * A person in the fun fact matcher application.
 * 
 * @author Jeremy Goecks
 * @version 0.1
 */
public class Person {
	
	/** List of valid hair colors. */
	public static final String[] VALID_HAIR_COLORS =
	{
		"blond",
		"brown",
		"black",
		"red",
		"gray",
		"other"		
	};
	
	/** List of valid eye colors. */
	public static final String[] VALID_EYE_COLORS =
	{
		"blue",
		"brown",
		"black",
		"hazel",
		"other"		
	};
	
	/** Person's name. */
	protected String name;
	
	/** File that is person's photo. */
	protected String photoFile;
	
	/** Person's hair color. */
	protected String hairColor;

	/** Person's eye color. */
	protected String eyeColor;
	
	/** Person's gender. */
	protected String gender;
	
	/** Does person have earrings? */
	protected boolean hasEarrings;
	
	/** Person's fun fact. */
	protected String funFact;
	
	/**
	 * Create a Person.
	 * @param name
	 * @param photoFile
	 * @param hairColor
	 * @param eyeColor
	 * @param gender
	 * @param answerHasEarrings
	 * @param funFact
	 */
	public Person(String name, String photoFile, String hairColor, 
					String eyeColor, String gender, boolean hasEarrings,
					String funFact) {
		this.name = name;
		this.photoFile = photoFile;
		this.hairColor = hairColor;
		this.eyeColor = eyeColor;
		this.gender = gender;
		this.hasEarrings = hasEarrings;
		this.funFact = funFact;
	} // end Person()
	
	/**
	 * @return Returns the gender.
	 */
	public String getGender() {
		return gender;
	}
	/**
	 * @return Returns the hairColor.
	 */
	public String getHairColor() {
		return hairColor;
	}
	/**
	 * @return Returns the answerHasEarrings.
	 */
	public boolean hasEarrings() {
		return hasEarrings;
	}
	/**
	 * @return Returns the name.
	 */
	public String getName() {
		return name;
	}
	/**
	 * @return Returns the photoFile.
	 */
	public String getPhotoFile() {
		return photoFile;
	}
	
	/**
	 * @return Returns the eyeColor.
	 */
	public String getEyeColor() {
		return eyeColor;
	}
	/**
	 * @return Returns the funFact.
	 */
	public String getFunFact() {
		return funFact;
	}
	
	/**
	 * Write person object to a file.
	 * @param outputFile file to write person to
	 * @return true if Person was written successfully; false otherwise
	 */
	public boolean writeToFile(File outputFile) throws IOException {
		//
		// File format is one item of data per line; end-of-line character is
		// '\n'. Thus a written file looks like this:
		//
		// [name]
		// [photoFile]
		// [hairColor]
		// [eyeColor]
		// [gender]
		// [hasEarrings]
		// [funFact]
		//
		//
		
		boolean success = false;
		
		// Create file writer.
		FileWriter fileOut = new FileWriter(outputFile);
		
		// Write name to file.
		fileOut.write(name + '\n');
		
		// Write photoFile to file.
		fileOut.write(photoFile + '\n');

		// Write hair color to file.
		fileOut.write(hairColor + '\n');

		// Write eye color to file.
		fileOut.write(eyeColor + '\n');

		// Write gender to file.
		fileOut.write(gender + '\n');
		
		// Write hasEarrings to file.
		fileOut.write(hasEarrings + "\n");

		// Write fun fact to file.
		fileOut.write(funFact + '\n');
		
		// Close writer.
		fileOut.close();
		
		// Success.
		success = true;
		
		return success;
	} // end writeToFile()
	
	/**
	 * Read a Person object from a file.
	 * @param inFile input file; this file is expected to be a file written by
	 * the writeToFile() method
	 * @return a Person object containing the data found from the file; null if
	 * file or data in file could not be read
	 */
	public static Person readFromFile(File inFile) throws IOException {
		//
		// Read person data from file.
		//
		Person person = null;
		
		// Create file reader; use a BufferedReader so that we can read
		// lines.
		FileReader fileIn = new FileReader(inFile);
		BufferedReader bufFileIn = new BufferedReader(fileIn);
		
		// Read name from file.
		String name = bufFileIn.readLine();
		
		// Error checking.
		if (name == null)
			throw new IOException("unexpected end of file; could not read name");
		
		// Read photoFile from file.
		String photoFile = bufFileIn.readLine();
		
		// Error checking.
		if (photoFile == null)
			throw new IOException("unexpected end of file; could not read photoFile");
		
		// Read hairColor from file.
		String hairColor = bufFileIn.readLine();
		
		// Error checking.
		if (hairColor == null)
			throw new IOException("unexpected end of file; could not read hairColor");
		
		// Read eyeColor from file.
		String eyeColor = bufFileIn.readLine();
		
		// Error checking.
		if (eyeColor == null)
			throw new IOException("unexpected end of file; could not read eyeColor");
		
		// Read gender from file.
		String gender = bufFileIn.readLine();
		
		// Error checking.
		if (gender == null)
			throw new IOException("unexpected end of file; could not read gender");
		
		// Read hasEarrings from file.
		String hasEarringsStr = bufFileIn.readLine();
		boolean hasEarrings = Boolean.valueOf(hasEarringsStr).booleanValue();
		
		// Error checking.
		if (gender == null)
			throw new IOException("unexpected end of file; could not read gender");
		
		// Read funFact from file.
		String funFact = bufFileIn.readLine();
		
		// Error checking.
		if (funFact == null)
			throw new IOException("unexpected end of file; could not read funFact");
					
		// Close reader.
		bufFileIn.close();
		
		// Create and return person.
		person = new Person(name, photoFile, hairColor, eyeColor, gender, hasEarrings, funFact);
		
		return person;
	} // end readFromFile()
	
	/**
	 * Returns a String representation of object suitable for debugging.
	 */
	public String toString() {
		return "Person(name=" + name + ",photoFile=" + photoFile + 
				",hairColor=" + hairColor + ",eyeColor=" + eyeColor +
				",gender=" + gender + ",hasEarrings=" + hasEarrings + 
				",funFact=" + funFact + ")";
	} // end toString()

} // end class Person

