import java.util.List;
import java.util.ArrayList;
import java.util.StringTokenizer;
import javax.swing.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;

/**
 * Class that represents a photo album.  It contains pictures and each picture
 * can have a caption.  The user can edit the catpions at any time.  When the
 * user closes the window the information is written to a file.  When you create
 * a photo album instance it will first check if there is an information file
 * in the directory and if so use that to create the album.  If there is not
 * an information file in the directory it will just display all pictures (.jpg
 * and .gif) in the directory with no captions.
 * 
 * @author Barb Ericson
 * Copyright 2004 Georgia Institute of Technology
 */
public class PhotoAlbum extends JFrame
{
  
  ///////////////////// fields /////////////////////////////////////
  
  /** a list of picture information objects */
  private PictureInfo[] pictureInfoArray = null;
 
  /** array of PicturePanel objects */
  private PicturePanel[] picturePanelArray;
  
  /** the number of rows in a page of the album */
  private int numRows = 3;
  
  /** the number of columns in a page of the album */
  private int numCols = 5;
  
  /** the name of the directory for this photo album */
  private String directoryName = null;
  
  /** the file name that contains picture file names and captions */
  private static String infoFileName = "album.txt";
  
  /** panel that is a page of the photo album */
  private JPanel page = new JPanel();
  
  /** the starting picture index on the page */
  private int startIndex = 0;
  
  /////////////////////// constructors /////////////////////////////
  
  /**
   * Constructor that takes a directory name
   * @param dir the name of the directory
   */
  public PhotoAlbum(String dir)
  {
    
    // set the directory field
    this.directoryName = dir;
    
    // try to load the information file from the given directory
    // put if there isn't one just load all files that have pictures
    // in them
    if (loadUsingInformationFile() == false)
      loadPictures();
    
    // show the pictures
    initialize();
    showPictures();
  }
  
  /**
   * Method to load the photo album from the picture information file
   * in the current directory
   */
  public boolean loadUsingInformationFile()
  {
    BufferedReader reader = null;
    PictureInfo pictureInfo = null;
    boolean resultFlag = true;
    String line = null;
    StringTokenizer tokenizer = null;
    String fileName = null;
    String caption = "";
    int index = 0;
    int numObjects = 0;
    
    // try to read from the file
    try {
      
      reader = new BufferedReader(new FileReader(directoryName + infoFileName));
      
      // read the number of picture information objects
      line = reader.readLine();
      numObjects = Integer.parseInt(line);
      pictureInfoArray = new PictureInfo[numObjects];
      
      // loop till end of line
      while ((line = reader.readLine()) != null)
      {
        tokenizer = new StringTokenizer(line,"|");
        fileName = tokenizer.nextToken();
        if (tokenizer.hasMoreTokens())
          caption = tokenizer.nextToken();
        else
          caption = "";
        pictureInfo = new PictureInfo(fileName,caption);
        pictureInfoArray[index++] = pictureInfo;
      }
      
      // close the reader
      reader.close();
      
    } catch (Exception ex) {
      resultFlag = false;
    }
    
    return resultFlag;
  }
  
  /**
   * Method to load the photo album from the current directory
   */
  public void loadPictures()
  {
    String[] pictureFileNames = null;
    String pictureFileName = null;
    PicturePanel picturePanel = null;
    PictureInfo pictureInfo = null;
    
    // check that the directory exists
    File file = new File(directoryName); 
    if (file.exists())
    {
      // get a list of image files in the directory
      pictureFileNames = file.list(new PictureFilter());
      
      // create the picture info array
      pictureInfoArray = new PictureInfo[pictureFileNames.length];
    
      // loop through the list and create the PicturePanel objects
      for (int i=0; i < pictureFileNames.length; i++)
      {
        pictureInfo = new PictureInfo(directoryName + pictureFileNames[i],"");
        pictureInfoArray[i] = pictureInfo;
      }
    }
  }
  
  /**
   * Method to initialize the panel
   */
  private void initialize()
  {
    int numPanels = numRows * numCols;
    PicturePanel picturePanel = null;
    int maxIndex = Math.min(numPanels,pictureInfoArray.length);
    JPanel mainPanel = new JPanel();
    
    // set up the interior panels
    JPanel buttonPanel = createButtonPanel();
    page.setLayout(new GridLayout(numRows,numCols));
    picturePanelArray = new PicturePanel[numPanels];
    for (int i = 0; i < maxIndex; i++)
    {
      picturePanel =  new PicturePanel(pictureInfoArray[i]);
      picturePanelArray[i] = picturePanel;
      page.add(picturePanel);
    }
    
    // set up this panel
    mainPanel.setLayout(new BorderLayout());
    mainPanel.add(page,BorderLayout.CENTER);
    mainPanel.add(buttonPanel,BorderLayout.SOUTH);
    this.getContentPane().add(mainPanel);
    this.addWindowListener(new MyWindowAdapter());
  }
  
  /**
   * Method to create the button panel
   */
  private JPanel createButtonPanel()
  {
    JPanel buttonPanel = new JPanel();
    buttonPanel.setLayout(new FlowLayout());
    JButton nextButton = new JButton("Next");
    nextButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e)
      {
        int next = startIndex + (numRows * numCols);
        if (next <= pictureInfoArray.length)
        {
          startIndex = next;
          showPictures();
        }
      }
    });
    JButton prevButton = new JButton("Previous");
    prevButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) 
      {
        int prev = startIndex - (numRows * numCols);
        if (prev >= 0)
        {
          startIndex = prev;
          showPictures();
        }
      }
    });
        
    // add the buttons to the button panel
    buttonPanel.add(prevButton);
    buttonPanel.add(nextButton);
    return buttonPanel;
  }
  
  /**
   * Method to show the pictures in a page of the photo album
   */
  public void showPictures()
  {
    int numPanels = numRows * numCols;
    int index = 0;
    
    // reset the picture info object associated with the 
    for (int i=0; i < numPanels; i++)
    {
      index = i + startIndex;
      if (index < pictureInfoArray.length)
        picturePanelArray[i].setPictureInfo((PictureInfo) pictureInfoArray[i + startIndex]);
      else
        picturePanelArray[i].clearPanel();
    }
        
    // repaint the panel
    repaint();
    
  }
  
  /** 
   * Class to check if a filename is a picture file 
   */
  private class PictureFilter implements FilenameFilter
  {
    /** 
     * Method to test if the file name is a picture file 
     * @param dir the directory name
     * @param name the file name
     * @return true if this file is a picture file, else false
     */
    public boolean accept(File dir, String name)
    {
      return (name.indexOf(".jpg") >= 0 || name.indexOf(".gif") >= 0);
    }

  }
  
  /**
   * Method to write out the picture information
   */
  public void writeInformation()
  {
    BufferedWriter writer = null;
    PictureInfo info = null;
    String endOfLine = System.getProperty("line.separator");
    
    // Try to write the picture information
    try {
      
      // create a writer 
      writer = new BufferedWriter(new FileWriter(directoryName + infoFileName));
      
      // write out the number of picture information objects
      writer.write(pictureInfoArray.length + endOfLine);
      
      // loop writing the picture info to the file
      for (int i = 0; i < pictureInfoArray.length; i++)
      {
        info = pictureInfoArray[i];
        writer.write(info.getFileName() + "|" + info.getCaption() + "|" + endOfLine);
      }
      
      // close the writer
      writer.close();
      
    } catch (Exception ex) {
      System.out.println("Trouble writing the picture information to " + infoFileName);
    }
  }
  
  /**
   * Main method for testing
   */
  public static void main(String[] argv)
  {
    String dir = "c:\\ICE-workshops\\intro-prog-java\\mediasources\\";
    PhotoAlbum photoAlbum = new PhotoAlbum(dir); // create a photo album with this directory
    photoAlbum.pack(); // set the window size to the contents size
    photoAlbum.setVisible(true); // show the window
  }
  
  /**
   * Private inner class to handle the window closing
   */
  private class MyWindowAdapter extends WindowAdapter
  {
    public void windowClosing(WindowEvent e) {
      writeInformation();
    }
  }
  
}