/*
 * File: CalendarDemo.java
 * -----------------------
 * This program uses the GUI layout mechanism to create
 * a calendar page.
 */

import acm.gui.*;
import acm.program.*;

import java.awt.*;
import java.awt.event.*;
import java.text.*;
import java.util.*;

import javax.swing.*;
import javax.swing.border.*;

/**
 * This class creates a calendar page for the current month.
 */
public class CalendarDemo extends Program implements ItemListener {

/** Set the program dimensions */
 public static final int APPLICATION_WIDTH = 700;
 public static final int APPLICATION_HEIGHT = 400;

/** Initialize the calendar display */
 public void init() {
  setBackground(Color.white);
  initCountryList();
  localeChooser = new JComboBox(countries);
  localeChooser.setSelectedItem(Locale.getDefault().getDisplayCountry());
  localeChooser.addItemListener(this);
  add(new JButton("<-"), NORTH);
  add(localeChooser, NORTH);
  add(new JButton("->"), NORTH);
  currentCalendar = Calendar.getInstance();
  itemStateChanged(null);
  addActionListeners();
 }

/** Create the calendar display */
 public void createCalendarDisplay(Calendar calendar) {
  setLayout(new TableLayout(0, 7, -1, -1));
  add(createMonthLabel(calendar), "gridwidth=7 bottom=3");
  for (int i = 0; i < 7; i++) {
   add(createWeekdayLabel(i), "weightx=1 width=1 bottom=2");
  }
  int weekday = getFirstWeekdayIndex(calendar);
  for (int i = 0; i < weekday; i++) {
   add(createEmptyDayBox(), "weighty=1");
  }
  int nDays = getDaysInMonth(calendar);
  for (int day = 1; day <= nDays; day++) {
   add(createDayBox(day), "weighty=1");
   weekday = (weekday + 1) % 7;
  }
  while (weekday != 0) {
   add(createEmptyDayBox(), "weighty=1");
   weekday = (weekday + 1) % 7;
  }
 }

 public void actionPerformed(ActionEvent e) {
  int delta = (e.getActionCommand().equals("<-")) ? -1 : +1;
  currentCalendar.add(Calendar.MONTH, delta);
  getRegionPanel(CENTER).removeAll();
  createCalendarDisplay(currentCalendar);
  validate();
 }

 public void itemStateChanged(ItemEvent e) {
  if (e == null || e.getStateChange() == ItemEvent.SELECTED) {
   Date time = currentCalendar.getTime();
   Locale locale = LOCALES[localeChooser.getSelectedIndex()];
   currentCalendar = Calendar.getInstance(locale);
   currentCalendar.setTime(time);
   symbols = new DateFormatSymbols(locale);
   weekdayNames = symbols.getWeekdays();
   monthNames = symbols.getMonths();
   firstDayOfWeek = currentCalendar.getFirstDayOfWeek();
   getRegionPanel(CENTER).removeAll();
   createCalendarDisplay(currentCalendar);
   validate();
  }
 }

 private JLabel createMonthLabel(Calendar calendar) {
  int month = calendar.get(Calendar.MONTH);
  int year = calendar.get(Calendar.YEAR);
  JLabel label = new JLabel(capitalize(monthNames[month]) + " " + year);
  label.setFont(TITLE_FONT);
  label.setHorizontalAlignment(JLabel.CENTER);
  return label;
 }

 private JLabel createWeekdayLabel(int index) {
  int weekday = (firstDayOfWeek + index + 6) % 7 + 1;
  JLabel label = new JLabel(capitalize(weekdayNames[weekday]));
  label.setFont(LABEL_FONT);
  label.setHorizontalAlignment(JLabel.CENTER);
  return label;
 }

 private Component createEmptyDayBox() {
  return createDayBox(null);
 }

 private Component createDayBox(int day) {
  return createDayBox("" + day);
 }

 private Component createDayBox(String text) {
  VPanel vbox = new VPanel();
  if (text == null) {
   vbox.setBackground(EMPTY_BACKGROUND);
  } else {
   JLabel label = new JLabel(text);
   label.setFont(DATE_FONT);
   vbox.add(label, "anchor=NORTHEAST top=2 right=2");
   vbox.setBackground(DATE_BACKGROUND);
  }
  vbox.setOpaque(true);
  vbox.setBorder(new LineBorder(Color.black));
  return vbox;
 }

 private int getFirstWeekdayIndex(Calendar calendar) {
  int day = calendar.get(Calendar.DAY_OF_MONTH);
  int weekday = calendar.get(Calendar.DAY_OF_WEEK);
  int weekdayIndex = (weekday + 7 - firstDayOfWeek) % 7;
  return ((5 * 7 + 1) + weekdayIndex - day) % 7;
 }

 private int getDaysInMonth(Calendar calendar) {
  return getActualMaximum(calendar, Calendar.DAY_OF_MONTH);
 }

 private int getActualMaximum(Calendar calendar, int field) {
  calendar = (Calendar) calendar.clone();
  int current = calendar.get(field);
  int next = current;
  while (next >= current) {
   current = next;
   calendar.add(field, 1);
   next = calendar.get(field);
  }
  return current;
 }

 private void initCountryList() {
  countries = new String[LOCALES.length];
  for (int i = 0; i < LOCALES.length; i++) {
   countries[i] = LOCALES[i].getDisplayCountry();
  }
 }

 private String capitalize(String word) {
  return word.substring(0, 1).toUpperCase() + word.substring(1).toLowerCase();
 }

/* Private constants */

 private static final Font TITLE_FONT = new Font("Times", Font.PLAIN, 36);
 private static final Font LABEL_FONT = new Font("Times", Font.BOLD, 14);
 private static final Font DATE_FONT = new Font("Times", Font.PLAIN, 18);

 private static final Color DATE_BACKGROUND = Color.WHITE;
 private static final Color EMPTY_BACKGROUND = new Color(0xDDDDDD);

/* Array of Locales to display in the JComboBox */

 private static final Locale[] LOCALES = {
  new Locale("fr", "FR", ""),
  new Locale("de", "DE", ""),
  new Locale("es", "MX", ""),
  new Locale("it", "IT", ""),
  new Locale("nl", "NL", ""),
  new Locale("es", "ES", ""),
  new Locale("en", "GB", ""),
  new Locale("en", "US", "")
 };

/* Private instance variables */

 private JComboBox localeChooser;
 private String[] countries;
 private Calendar currentCalendar;
 private DateFormatSymbols symbols;
 private String[] monthNames;
 private String[] weekdayNames;
 private int firstDayOfWeek;
 
 public static void main(String[] args)
 {
   CalendarDemo demo = new CalendarDemo();
   demo.start();
 }
}
