import java.io.*;

/**
 * Class used to write web (HTML) pages
 * @author Barb Ericson
 */
public class WebPageWriter
{
  
  /**
   * Method to write an example web (HTML) page
   * @param fileName the file to write to
   */
  public void writeExamplePage(String fileName) 
  {
    // try the following
    try {
      
      // open a file for writing 
      BufferedWriter writer =
        new BufferedWriter(new FileWriter(fileName));
      
      // start the html page
      writer.write("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD " +
                   "HTML 4.01 Transition//EN\"");
      writer.newLine();
      writer.write("\"http://www.w3.org/TR/html4/loose.dtd\">");
      writer.newLine();
      writer.write("<html>");
      writer.newLine();
      
      // write out the header
      writer.write("<head><title>A simple web page" +
                   "</title></head>");
      writer.newLine();
      
      // write out the body
      writer.write("<body>");
      writer.newLine();
      writer.write("<h1>A Simple Heading</h1>");
      writer.newLine();
      writer.write("<p>Some simple text</p>");
      writer.newLine();
      writer.write("</body>");
      writer.newLine();
      
      // end the page
      writer.write("</html>");
      
      // close the writer
      writer.close();
      
    } catch (Exception ex) {
      ex.printStackTrace();
    }
    
  }
  
  /**
   * Method that will write out a homepage for a person
   * with the passed name
   * @param name the name of the person that this homepage
   * is for
   * @param interests a list of the person's interests
   */
  public void writeHomepage(String name, String interests)
  {
    // try the following
    try {
      
      // open a file for writing 
      BufferedWriter writer =
        new BufferedWriter(new FileWriter(name + ".html"));
      
      // start the html page
      writer.write("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD " +
                   "HTML 4.01 Transition//EN\"");
      writer.newLine();
      writer.write("\"http://www.w3.org/TR/html4/loose.dtd\">");
      writer.newLine();
      writer.write("<html>");
      writer.newLine();
      
      // write out the header
      writer.write("<head><title>" + name + 
                   "'s Homepage</title></head>");
      writer.newLine();
      
      // write out the body
      writer.write("<body>");
      writer.newLine();
      writer.write("<h1>Welcome to " + name + 
                   "'s Homepage</h1>");
      writer.newLine();
      writer.write("<p>I am interested in " + 
                   interests + "</p>");
      writer.newLine();
      writer.write("</body>");
      writer.newLine();
      
      // end the page
      writer.write("</html>");
      
      // close the writer
      writer.close();
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }
  
  /**
   * Method to write the doctype and html tags
   * @param writer the writer to use
   * @throws IOException 
   */
  private void writeStart(BufferedWriter writer) throws
    IOException
  {
    // write the document type
    writer.write("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD " +
                 "HTML 4.01 Transition//EN\"");
    writer.newLine();
    writer.write("\"http://www.w3.org/TR/html4/loose.dtd\">");
    writer.newLine();
    
    // start the html page
    writer.write("<html>");
    writer.newLine();
  }
  
  /**
   * Method to write the title out
   * @param writer the writer to use
   * @param title the title to use
   * @throws IOException
   */
  private void writeHead(BufferedWriter writer, String title) 
    throws IOException
  {
    writer.write("<head><title>" + title + "</title></head>");
    writer.newLine();
  }
  
  /**
   * Method to write the body of the page
   * @param writer the writer to use
   * @param body the body to write
   * @throws IOException
   */
  private void writeBody(BufferedWriter writer, String body)
    throws IOException
  {
    writer.write("<body>" + body + "</body>");
    writer.newLine();
  }
  
  /**
   * Method to finish the html page
   * @param writer the writer to use
   * @throws IOException
   */
  private void writeEnd(BufferedWriter writer)
    throws IOException
  {
    writer.newLine();
    writer.write("</body>");
    writer.newLine();
    writer.write("</html>");
  }
  
  /**
   * Method for writing a homepage for the passed
   * name
   * @param name the person's name 
   * @param interests a list of the person's interests
   */
  public void writeHomepageV2(String name, String interests)
  {
    
    // try the following
    try {
      
      // open a file for writing 
      BufferedWriter writer =
        new BufferedWriter(new FileWriter(name + ".html"));
      
      // write the start
      writeStart(writer);
      
      // write the header
      writeHead(writer,name + "'s Homepage");
      
      // write the body
      writeBody(writer,"<h1>Welcome to " + name + 
                "'s Homepage</h1>" +
                "<p> I am interested in " + interests);
      
      // end the page
      writeEnd(writer);
      
      // close the writer
      writer.close();
      
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }
  
  /**
   * Method to create a page with thumbnails of all the images
   * in a directory
   * @param directory the directory to create the page for
   */
  public void createImagePage(String directory)
  {
    String name = null;
    String body = "";
    String endOfLine = System.getProperty("line.separator");
    
    // try the following 
    try {
      
      // create the File object
      File dir = new File(directory);
      
      // get the full path name of the directory
      String pathName = directory + dir.getName() + ".html";
      BufferedWriter writer =
        new BufferedWriter(new FileWriter(pathName));
      
      // write the start
      writeStart(writer);
      
      // write the head
      writeHead(writer,"Thumbnails from " + directory);
      
      // get the array of items in the directory
      String[] items = dir.list();
      
      // loop through the array
      for (int i = 0; i < items.length; i++)
      {
        name = items[i];
        if (name.indexOf(".jpg") >= 0)
        {
          body = body + "<p>Filename: " + name + 
            "<img src='" + name + "' height='150'/></p>" + 
            endOfLine;
        }
      }
      
      // write the body
      writeBody(writer,body);
      
      // write the end
      writeEnd(writer);
      
      // close the writer
      writer.close();
      
    } catch (Exception ex) {
      ex.printStackTrace();
    }                       
  }
  
  /**
   * Method to create a page with links to the sound files
   * in the passed directory
   * @param directory the directory to create the page for
   */
  public void createSoundPage(String directory)
  {
    String name = null;
    String body = "";
    String endOfLine = System.getProperty("line.separator");
    
    // try the following 
    try {
      
      // create the File object
      File dir = new File(directory);
      
      // get the full path name of the directory
      String pathName = directory + dir.getName() + "S.html";
      BufferedWriter writer =
        new BufferedWriter(new FileWriter(pathName));
      
      // write the start
      writeStart(writer);
      
      // write the head
      writeHead(writer,"Sounds from " + directory);
      
      // get the array of items in the directory
      String[] items = dir.list();
      
      // loop through the array
      for (int i = 0; i < items.length; i++)
      {
        name = items[i];
        if (name.indexOf(".wav") >= 0)
        {
          body = body + "<p><a href=" + name + ">" +  name + 
            "</a></p>" + 
            endOfLine;
        }
      }
      
      // write the body
      writeBody(writer,body);
      
      // write the end
      writeEnd(writer);
      
      // close the writer
      writer.close();
      
    } catch (Exception ex) {
      ex.printStackTrace();
    }                       
  }
  
  /**
   * Method for writing a homepage for the passed
   * name that displays the current weather
   * @param name the person's name
   * @param interests a list of the person's interests
   */
  public void writeHomepageV3(String name,
                              String interests)
  {
    // get the current temperature
    TempFinder tempFinder = new TempFinder();
    String urlString = "http://www.ajc.com/";
    String currTemp = tempFinder.getTempFromNetwork(urlString);

    // try the following
    try {

      // open a file for writing
      BufferedWriter writer =
        new BufferedWriter(new FileWriter(name + ".html"));

      // write the start
      writeStart(writer);

      // write the header
      writeHead(writer,name + "'s Homepage");

      // write the body
      writeBody(writer,"<h1>Welcome to " + name +
                "'s Homepage</h1>" +
                "<p> I am interested in " + interests +
                ".</p><p>Right now it is " + currTemp + 
                " degrees.</p>");

      // end the page
      writeEnd(writer);

      // close the writer
      writer.close();

    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }
  
  /**
   * Method for writing a homepage for the passed
   * name that displays the current weather and a
   * random sentence
   * @param name the person's name 
   * @param interests a list of the person's interests
   */
  public void writeHomepageV4(String name, 
                              String interests)
  {
    // get the current temperature
    TempFinder tempFinder = new TempFinder();
    String urlString = "http://www.ajc.com/";
    String currTemp = tempFinder.getTempFromNetwork(urlString);
    
    // get the random sentence
    SentenceGenerator sentenceGen = new SentenceGenerator();
    String sentence = sentenceGen.generateRandomSentence();
    
    // try the following
    try {
      
      // open a file for writing 
      BufferedWriter writer =
        new BufferedWriter(new FileWriter(name + ".html"));
      
      // write the start
      writeStart(writer);
      
      // write the header
      writeHead(writer,name + "'s Homepage");
      
      // write the body
      writeBody(writer,"<h1>Welcome to " + name + 
                "'s Homepage</h1>" +
                "<p> I am interested in " + interests +
                ".</p><p>Right now it is " + currTemp + 
                " degrees.</p>" +
                "<p>The random thought for the day is:  " +
                sentence + "</p>");
      
      // end the page
      writeEnd(writer);
      
      // close the writer
      writer.close();
      
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }
  
  /**
   * Method for writing a homepage for the passed
   * first name that displays her/his interests and age
   * @param name the person's first name
   * @param interests a list of the person's interests
   */
  public void writeHomepageV5(String name, 
                              String interests)
  {
    // Get a DatabaseManager object 
    DatabaseManager dbManager = 
      new DatabaseManager("sun.jdbc.odbc.JdbcOdbcDriver",
                          "jdbc:odbc:person");
    
    // get this person's age
    String age =
      dbManager.getStringForQuery(
         "Select Age From Person " +
         "Where FirstName='" + name + "'");
    
    // try the following
    try {
      
      // open a file for writing 
      BufferedWriter writer =
        new BufferedWriter(new FileWriter(name + ".html"));
      
      // write the start
      writeStart(writer);
      
      // write the header
      writeHead(writer,name + "'s Homepage");
      
      // write the body
      writeBody(writer,"<h1>Welcome to " + name + 
                "'s Homepage</h1>" +
                "<p> I am interested in " + interests +
                ".</p><p>I am " + age + " years old</p>");
      
      // end the page
      writeEnd(writer);
      
      // close the writer
      writer.close();
      
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }
  
  public static void main(String[] args)
  {
    WebPageWriter writer = new WebPageWriter();
//    writer.writeExamplePage("simple.html");
//    writer.writeHomepage("Mark", "reading");
//    writer.writeHomepageV2("Barb", "horseback riding");
    String dir = "c:/intro-prog-java/mediasources/";
//    writer.createImagePage(dir);
   // writer.writeHomepageV5("Matthew","playing video games");
    writer.createSoundPage(dir);
  }
}